import hashlib
import base64
import ipaddress
from fire import Fire


def htpasswd(user, password, filepath=None):
    # from https://httpd.apache.org/docs/2.4/misc/password_encryptions.html
    hashed_password = base64.b64encode(
        hashlib.sha1(password.encode()).digest()
    ).decode()
    token = f"{user}:{{SHA}}{hashed_password}"
    if filepath is not None:
        with open(filepath, "w") as file_pointer:
            file_pointer.write(token + "\n")

    return token


def ip_in_cidr(cidr, following=None):

    condition = following is None
    for i, ip in enumerate(ipaddress.ip_network(cidr)):
        if i == 0:
            continue

        if condition:
            return str(ip)

        if following == str(ip):
            condition = True

    return None


if __name__ == "__main__":
    Fire()



import tempfile
from traxible.traxodule import Traxodule
from traxible.utils import htpasswd


class Nginx(Traxodule):
    def install(self):
        self.run(["sudo apt install -y nginx certbot python3-certbot-nginx"])

    def add_docker(self, domain, docker_ip, basic_auth=None):

        basic_auth_config = ""
        if basic_auth is not None:
            htpass_file = tempfile.NamedTemporaryFile(mode="w")
            htpasswd(
                user=basic_auth["user"],
                password=basic_auth["password"],
                filepath=htpass_file.name,
            )
            basic_auth_config = f"""
auth_basic           "Administrator’s Area";
auth_basic_user_file "/etc/nginx/{domain}";
"""

            self.put(htpass_file.name, f"/etc/nginx/{domain}", sudo=True)

        new_conf = f"""
server {{
    index index.html index.htm;
    server_name {domain};
    
    location / {{
        {basic_auth_config}
        proxy_pass http://{docker_ip}:4242;
        proxy_redirect off;
        proxy_set_header Host $host;
    }}

    listen [::]:443 ssl ipv6only=on;
    listen 443 ssl;

}}
"""
        with tempfile.NamedTemporaryFile(mode="w") as nginx_conf:
            nginx_conf.write(new_conf)
            nginx_conf.flush()
            self.put(nginx_conf.name, f"/etc/nginx/sites-enabled/{domain}", sudo=True)

    def certbot(self, domain, mail):
        self.run(
            [
                f"sudo certbot --nginx  --non-interactive"
                f"--redirect --agree-tos -m {mail} --domains {domain}"
            ]
        )

    def restart(self):
        self.run(["sudo service nginx restart"])



from fire import Fire

from traxible.traxible import Traxible
from traxible.docker import Docker
from traxible.deb import Deb
from traxible.nginx import Nginx
from traxible.utils import ip_in_cidr


DEFAULT_PACKAGES = "tree ncdu fail2ban vim nano grep tmux"


def setup_web_site(
    self,
    host,
    network_name="my_network",
    cidr="172.42.0.0/16",
    domain="example.com",
    mail="no-reply@exmaple.com",
):
    traxible = Traxible(host)

    # setup machine
    deb = Deb(traxible)
    deb.update()
    deb.install(DEFAULT_PACKAGES)

    # install docker
    docker = Docker(traxible)
    docker.install()
    traxible.reconnect()
    docker.list()
    docker.network(network_name, cidr)
    docker_ip = ip_in_cidr(cidr)
    docker.create(
        name="my_docker",
        image="my_image:latest",
        network_name=network_name,
        port_expose=8080,
        ip=docker_ip,
    )

    # install and configure nginx
    nginx = Nginx(traxible)
    nginx.install()
    nginx.add_docker(domain, docker_ip)
    nginx.certbot(domain, mail)
    nginx.restart()


if __name__ == "__main__":
    Fire()



from setuptools import setup

with open("requirements.txt") as f:
    required = f.read().splitlines()

    setup(
        name="traxible",
        version="14",
        url="https://gitlab.com/traxix/traxible",
        packages=["traxible"],
        install_requires=required,
        license="GPLv3",
        author="trax Omar Givernaud",
    )



import tempfile
from fire import Fire
from fabric import Connection


class Traxible:
    """
    This is main module that allows basic remote operations
    sush as ssh connection, send command or files, reconnect, ...

    """

    def __init__(self, host, dry_run=False):
        self.host = host
        self.conn = Connection(self.host)
        self.dry_run = dry_run

    def run(self, commands, block=True):
        for command in commands:
            print("> ", command)
            if self.dry_run:
                continue
            try:
                result = self.conn.run(command)
                print(result.exited, "=" * 60)

            except Exception as error:
                if block is True:
                    raise error

    def put(self, source, destination, sudo=False):
        if self.dry_run:
            print(f"> cp {source} {destination}")
            return

        if sudo is False:
            return self.conn.put(source, destination)

        self.conn.put(source, "/tmp/efe9e3023a502013767a0559a94a5e")
        return self.run([f"sudo mv /tmp/efe9e3023a502013767a0559a94a5e {destination}"])

    def write(self, source, destination, sudo=False):
        with tempfile.NamedTemporaryFile(mode="w") as temp_file:
            temp_file.write(source)
            self.put(source=temp_file.name, destination=destination, sudo=sudo)

    def reconnect(self):
        self.conn.close()
        self.conn.open()


if __name__ == "__main__":
    Fire(Traxible)
